@@ -53,13 +53,16 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
+
 import org.eclipse.jgit.junit.TestRng;
+import org.eclipse.jgit.util.TemporaryBuffer.Block;
 import org.junit.Test;
 
 public class TemporaryBufferTest {
 	@Test
 	public void testEmpty() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		try {
 			b.close();
 			assertEquals(0, b.length());
@@ -73,7 +76,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testOneByte() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte test = (byte) new TestRng(getName()).nextInt();
 		try {
 			b.write(test);
@@ -85,10 +89,8 @@ public class TemporaryBufferTest {
 				assertEquals(1, r.length);
 				assertEquals(test, r[0]);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(1, r.length);
 				assertEquals(test, r[0]);
@@ -100,7 +102,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testOneBlock_BulkWrite() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.Block.SZ);
 		try {
@@ -116,10 +119,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -131,7 +132,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testOneBlockAndHalf_BulkWrite() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.Block.SZ * 3 / 2);
 		try {
@@ -147,10 +149,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -162,7 +162,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testOneBlockAndHalf_SingleWrite() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.Block.SZ * 3 / 2);
 		try {
@@ -176,10 +177,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -191,7 +190,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testOneBlockAndHalf_Copy() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.Block.SZ * 3 / 2);
 		try {
@@ -206,10 +206,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -221,7 +219,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testLarge_SingleWrite() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.DEFAULT_IN_CORE_LIMIT * 3);
 		try {
@@ -234,10 +233,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -251,19 +248,19 @@ public class TemporaryBufferTest {
 	public void testInCoreInputStream() throws IOException {
 		final int cnt = 256;
 		final byte[] test = new TestRng(getName()).nextBytes(cnt);
-		final TemporaryBuffer.Heap b = new TemporaryBuffer.Heap(cnt + 4);
-		b.write(test);
-		b.close();
-
-		InputStream in = b.openInputStream();
-		byte[] act = new byte[cnt];
-		IO.readFully(in, act, 0, cnt);
-		assertArrayEquals(test, act);
+		try (TemporaryBuffer.Heap b = new TemporaryBuffer.Heap(cnt + 4)) {
+			b.write(test);
+			InputStream in = b.openInputStream();
+			byte[] act = new byte[cnt];
+			IO.readFully(in, act, 0, cnt);
+			assertArrayEquals(test, act);
+		}
 	}
 
 	@Test
 	public void testInCoreLimit_SwitchOnAppendByte() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.DEFAULT_IN_CORE_LIMIT + 1);
 		try {
@@ -277,10 +274,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -292,7 +287,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testInCoreLimit_SwitchBeforeAppendByte() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.DEFAULT_IN_CORE_LIMIT * 3);
 		try {
@@ -306,10 +302,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -321,7 +315,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testInCoreLimit_SwitchOnCopy() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final byte[] test = new TestRng(getName())
 				.nextBytes(TemporaryBuffer.DEFAULT_IN_CORE_LIMIT * 2);
 		try {
@@ -338,10 +333,8 @@ public class TemporaryBufferTest {
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(test.length, r.length);
 				assertArrayEquals(test, r);
@@ -353,7 +346,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testDestroyWhileOpen() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		try {
 			b.write(new TestRng(getName())
 					.nextBytes(TemporaryBuffer.DEFAULT_IN_CORE_LIMIT * 2));
@@ -364,7 +358,8 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testRandomWrites() throws IOException {
-		final TemporaryBuffer b = new TemporaryBuffer.LocalFile();
+		@SuppressWarnings("resource") // Buffer is explicitly destroyed in finally block
+		final TemporaryBuffer b = new TemporaryBuffer.LocalFile(null);
 		final TestRng rng = new TestRng(getName());
 		final int max = TemporaryBuffer.DEFAULT_IN_CORE_LIMIT * 2;
 		final byte[] expect = new byte[max];
@@ -396,10 +391,8 @@ public class TemporaryBufferTest {
 				assertEquals(expect.length, r.length);
 				assertArrayEquals(expect, r);
 			}
-			{
-				final ByteArrayOutputStream o = new ByteArrayOutputStream();
+			try (ByteArrayOutputStream o = new ByteArrayOutputStream()) {
 				b.writeTo(o, null);
-				o.close();
 				final byte[] r = o.toByteArray();
 				assertEquals(expect.length, r.length);
 				assertArrayEquals(expect, r);
@@ -411,6 +404,7 @@ public class TemporaryBufferTest {
 
 	@Test
 	public void testHeap() throws IOException {
+		@SuppressWarnings("resource" /* java 7 */)
 		final TemporaryBuffer b = new TemporaryBuffer.Heap(2 * 8 * 1024);
 		final byte[] r = new byte[8 * 1024];
 		b.write(r);
@@ -422,4 +416,27 @@ public class TemporaryBufferTest {
 			assertEquals("In-memory buffer limit exceeded", e.getMessage());
 		}
 	}
+
+	@Test
+	public void testHeapWithEstimatedSize() throws IOException {
+		int sz = 2 * Block.SZ;
+		try (TemporaryBuffer b = new TemporaryBuffer.Heap(sz / 2, sz)) {
+			for (int i = 0; i < sz; i++) {
+				b.write('x');
+			}
+			try {
+				b.write(1);
+				fail("accepted too many bytes of data");
+			} catch (IOException e) {
+				assertEquals("In-memory buffer limit exceeded", e.getMessage());
+			}
+
+			try (InputStream in = b.openInputStream()) {
+				for (int i = 0; i < sz; i++) {
+					assertEquals('x', in.read());
+				}
+				assertEquals(-1, in.read());
+			}
+		}
+	}
 }
